% minimize alpha*TV(Phi'*x) + beta*||x||_1 + 0.5*||Ax-b||_2^2
%
% Authors: Shiqian Ma, Wotao Yin
% Date : 10/16/2007
% Last Modified: 08/17/2008
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clc; close all; clear all;
addpath('Rice_Wavelet_Toolbox_2.4');
rand('state',0); randn('state',0);
% F = double(imread('pics/testb.png'));
% F = double(imread('pics/MRI_Coronal_Brain.jpg'));
% F = double(imread('pics/3DMR_Chest.jpg'));
F = double(imread('pics/3DMR_Renal_Arteries.jpg'));
% F = double(imread('pics/MRI_Full_Body.jpg'));
% F = 10.0*phantom(256);
[m n] = size(F); N = m*n;
f = F(:);

d = 2;
[OMEGA] = RandMask_rect(double(m/d),double(n/d),m,n);
% Number of projections
k = length(OMEGA);

% partial FFT tranform
R = @(x) A_fhp_rect(x, OMEGA, m, n);
RT = @(x) At_fhp_rect(x, OMEGA, m, n);


% define the function handles that compute 
% the products by W (inverse DWT) and W' (DWT)
wav = daubcqf(2);
W = @(x) midwt(x,wav);
WT = @(x) mdwt(x,wav);

AO = @(x) R(W(x)); AOT = @(x) WT(RT(x)); 
A = A_operator(@(x) AO(x), @(x) AOT(x));
Phi = A_operator(@(x) WT(x), @(x) W(x));    % notice Phi=WT

% Measurement noise standard deviation and noise samples
sigma = 0.01; noise = sigma*randn(2*k+1,1);

% generate observations
b = R(f) + noise;

% parameters
alpha = 1e-3; beta = 3.5e-2; 
% Call the function
input.n1=m;
input.n2=n;
input.alpha=alpha;
input.beta=beta;
input.A=A;
input.Phi=Phi;
input.b=b;

xs = WT(f);
fprintf('calling the function TVCMRI.....\n');
out = TVCMRI(input,...
               'true x',xs,...
               'print level',1);
fprintf('TVCMRI terminated.\n\n');

x = out.x;
snr_1 = snr(W(x),f);
fprintf(1,'Init_time = %.2fsec, Iter_time = %.2fsec, ',out.init_time, out.iter_time);
fprintf(1,'k = %d, rel_err = %10.6e, samp.ratio = %f, snr = %f\n\n',...
    k, norm(x-xs)/norm(xs),k/(m*n),snr_1);

figure(1);
scrsz = get(0,'ScreenSize');
set(1,'Position',[10 scrsz(4)*0.1 scrsz(3)*0.3 0.65*scrsz(4)])
subplot(2,1,1)
imagesc(F);
colormap(gray)
axis image
axis equal
axis off
title(sprintf('Original image (%d x %d = %d pixels)',m,n,m*n))

subplot(2,1,2)
imagesc(reshape(W(x),m,n));
colormap(gray)
axis image
axis equal
axis off
title(sprintf('Reconstruction (k = %d projections)',k));